#!/usr/bin/perl -w
#
# Add flag to digiKam's XMP for DarkTable
# Copyright (C) 2015 Beat Rubischon
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Add DarkTable's namespace and set auto_presets_applied to 0
# to digiKam's generated XMP files. Without this flag DarkTable would
# assume the images were processed by Lightroom and ignores
# auto presets.
#
# 27.9.15 Beat Rubischon <beat@0x1b.ch>
#         Initial version
#
use strict;
use Fcntl qw(:flock);

while(my $xmp=shift) {
  my @newxml;
  my $inns=0;
  my $nsout=0;
  my $tagout=0;
  my $rewrite=1;
  print STDERR "Processing $xmp: ";
  open IN, $xmp;
  flock(IN, LOCK_EX);
  while(<IN>) {
    chomp;
    if (/xmlns:darktable="http:\/\/darktable.sf.net\/"/) {
      $rewrite=0;
    }
    if (! $tagout) {
      if (! $nsout) {
        if (/^    xmlns:/) {
          $inns=1;
        } else {
          if ($inns) {
            $inns=0;
            push @newxml, "    xmlns:darktable=\"http://darktable.sf.net/\"";
            $nsout=1;
          }
        }
      } else {
        if (/(.*)>$/) {
          $_=$1;
          $tagout=1;
        }
      }
    }
    push @newxml, "$_";
    if ($tagout == 1) {
      push @newxml, "   darktable:auto_presets_applied=\"0\">";
      $tagout=2;
    }
  }
  if ($rewrite) {
    open OUT, ">".$xmp.".tmp";
    foreach (@newxml) {
      print OUT "$_\n";
    }
    close OUT;
#    rename $xmp, $xmp."~";
    unlink $xmp;
    rename $xmp.".tmp", $xmp;
    print STDERR "updated\n";
  } else {
    print STDERR "unchanged\n";
  }
  flock(IN, LOCK_UN);
  close IN;
}
