#!/usr/bin/perl -w
#
# Image stacker for digiKam
# Copyright (C) 2015 Beat Rubischon
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Logic:
# Oldest image becomes the root picture, as digiKam never writes to
# RAW files they are always on the bottom. Newest image is on the top.
# The script takes into account that suffixes are added to images.
#
# Example:
# CT9C1234.jpeg		an image out of Gimp, latest timestamp
# CT9C1234_rt.jpg	copy out of RawTherapee
# CT9C1234_lr.jpg	something I did in Lightroom
# CT9C1234.DNG		conversion to DNG
# CT9C1234.CR2		original file
#
# Only online images are processed to speed up regular use
# stacks are not reordered as long as ImageRelations is not cleared
#
# 30.9.15 Beat Rubischon <beat@0x1b.ch>
#         Initial version
# 7.10.15 Beat Rubischon <beat@0x1b.ch>
#         Set album date to the date in the foldername
#         (YYYYMMDD-something-random)
#
# 6.1.16  Beat Rubischon <beat@0x1b.ch>
#         Fix albumfix to create valid dates
#
use strict;
use DBI;

$|=1;

my $DKDB="/scratch/Pictures/Library/digikam4.db";
my $dbh=DBI->connect("dbi:SQLite:dbname=$DKDB","","");

my $albums=0;
my $fixedalbums=0;
my $pictures=0;
my $stacks=0;
my $members=0;
my $newmembers=0;

print "Stacking Images...\n";

my $albumquery=$dbh->prepare("select t1.id, t2.identifier, t1.relativePath, \
  t1.date from Albums as t1, AlbumRoots as t2 where t1.albumRoot = t2.id");
$albumquery->execute();
my $albumfix=$dbh->prepare("update Albums set date=? where id=?");
while (my $album=$albumquery->fetch) {
  my $albumid=$album->[0];
  my $identifier=$album->[1];
  my $relativePath=$album->[2];
  my $date=$album->[3];

  if ($relativePath=~/^\/\d{4}\/\d{2}\/(\d{4})(\d{2})(\d{2})-.*/) {
    my $y=$1;
    my $m=$2;
    my $d=$3;
    if ($d eq "00") {
      $d="01";
    }
    if ($date ne $y."-".$m."-".$d) {
      $albumfix->execute($y."-".$m."-".$d, $albumid);
      $fixedalbums++;
    }
  }

  $identifier=~s/^volumeid:\?path=//;
  $identifier=~s/^networkshareid:\?mountpath=//;
  $identifier=~s/%2F/\//g;
  
  #print $albumid." ".$identifier.$relativePath."\n";
  $albums++;

  if (! -d $identifier.$relativePath) {
    next;
  }

  my %rootpics;
  my %rootlen;
  my %group;

  my $imgquery=$dbh->prepare("select id, name, modificationDate from Images where album=? order by modificationDate");
  my $linkquery=$dbh->prepare("select subject, object, type from ImageRelations where subject=? and object=? and type=?");
  my $linkinsert=$dbh->prepare("insert into ImageRelations values(?, ?, ?)");
  my $linkdelobj=$dbh->prepare("delete from ImageRelations where object=? and type=?");

  $imgquery->execute($albumid);
  while(my $row=$imgquery->fetch) {
    my $id=$row->[0];
    my $name=$row->[1];
    my $modificationDate=$row->[2];
    my $rootid;

    $pictures++;

    if ($name=~/^(.*)\.([^\.]+)/) {
      my $base=$1;
      my $ext=$2;
      my $len=length($base);
      my $matchlen;

      foreach my $i (sort {$b <=> $a} keys %rootlen) {
        if (defined $rootpics{substr($base, 0, $i)}) {
          $rootid=$rootpics{substr($base, 0, $i)};
          $matchlen=$i;
          last;
        }
      }

#      if (! defined $rootid or $len > $matchlen) {
      if (! defined $rootid) {
        $rootpics{$base}=$id;
        $rootlen{$len}=1;
      }
    }

    if (defined $rootid) {
      #print "  $name\t$id derivate of $rootid ";
      $linkquery->execute($id, $rootid, 1);
      my $row=$linkquery->fetch;
      if (defined $row->[0]) {
        #print "linked\n"
      } else {
        #print "link\n";
        $linkinsert->execute($id, $rootid, 1);
        #&copytags($rootid, $id);
      }
      $linkquery->finish;

      if (defined $group{$rootid}) {
        $group{$rootid}.=",".$id;
      } else {
        $group{$rootid}=$rootid.",".$id;
      }
    }
  }
  $imgquery->finish;

  foreach my $rootid (keys %group) {
    #print "  ".$group{$rootid}."\n";
    $stacks++;
    $members++;
    my @ids=split(/,/, $group{$rootid});
    my $topid=pop(@ids);
    while (my $id=pop(@ids)) {
      $linkdelobj->execute($id, 2);
      #print "    $id behind $topid ";
      $members++;
      $linkquery->execute($id, $topid, 2);
      my $row=$linkquery->fetch;
      if (defined $row->[0]) {
        #print "stacked\n";
      } else {
        #print "stacking\n";
        $newmembers++;
        $linkinsert->execute($id, $topid, 2);
      }
    $linkquery->finish;
    }
  }
}

$albumquery->finish;
$dbh->disconnect;

print "$albums albums with $pictures images, ";
print $pictures - $members + $stacks;
print " root images\n";
print "$stacks stacks with $members members, $newmembers newly stacked\n";
print "$fixedalbums albums got new date\n";
