#!/usr/bin/perl -w
#
# Copy resized preview and add metadata to images out of RawTherapee
#
# 27.9.15  Beat Rubischon <beat@0x1b.ch>
#          Initial version
# 7.10.15  Beat Rubischon <beat@0x1b.ch>
#          Folder Notify sometimes looses events, look through
#          directory added. Also handling .out.pp3 files from
#          RawTherapee
# 14.10.15 Beat Rubischon <beat@0x1b.ch>
#          Sleep until the .pp3 file is written to avoid race condition
# 12.2.16  Revert to loop through filelist and remove lockfile - Linux's
#          incron is more reliable, but will be triggered by lockfile and
#          ends in an endless loop
# 6.7.18   Add tif as source files for EXIF data
# 7.7.18   Revert change from 12.2.16 since RawThreapee delivers scanned
#          images too fast
use strict;

my $os=`/usr/bin/uname`;
chomp $os;

my $exiftool="/usr/bin/exiftool";
my $convert="/usr/bin/convert";
my $cp="/usr/bin/cp";

if ($os eq "Darwin") {
  $exiftool="/opt/local/bin/exiftool";
  $convert="/opt/local/bin/convert";
  $cp="/bin/cp";
}

#open LOG, ">>/scratch/Pictures/Scripts/todk.log";

open LCK, ">/scratch/Pictures/Convert/.lock";
flock(LCK, 2) || die "Cant't lock";

#while (my $infile=shift) {

opendir DIR, "/scratch/Pictures/Convert";
while(my $infile=readdir(DIR)) {
  $infile="/scratch/Pictures/Convert/".$infile;

  #print LOG "-" x 78;
  #print LOG "\n";
  #print LOG "$infile\n\n";

  if ($infile=~/([^\/]+)\$(.+)\.([^\.]+)$/) {
    my $folder=$1;
    my $file=$2;
    my $ext=$3;

    if ($ext eq "pp3") {
      next;
    }

    while (! -f $infile.".out.pp3") {
      sleep 1;
    }
    sleep 1;

    my $preview="/scratch/Pictures/Library/".
      substr($folder, 0, 4)."/".
      substr($folder, 4, 2)."/".
      $folder."/".
      $file."_rt.jpg";
    my $outfile="/scratch/Pictures/Temp/".$file.".".$ext;

    my $tagfile;
    if (-f $preview) {
      $tagfile=$preview;
    } else {
      my $i=length($file);
      while($i > 0 and ! defined $tagfile) {
        my $testfile="/scratch/Pictures/Library/".
          substr($folder, 0, 4)."/".
          substr($folder, 4, 2)."/".
          $folder."/".
          substr($file, 0, $i);
        foreach my $rawext (qw(CR2.xmp cr2.xmp TIFF tiff TIF tif JPEG jpeg JPG jpg)) {
          if (-f $testfile.".".$rawext) {
            $tagfile=$testfile.".".$rawext;
            last;
          }
        }
        $i--;
      }
    }

    #print LOG "$infile\n$tagfile\n$outfile\n$preview\n";

    rename $infile, $outfile;
    rename $infile.".out.pp3", $outfile.".out.pp3";
    system $exiftool, "-overwrite_original",
      "-tagsFromFile", $tagfile,
      "-Orientation=Horizontal", $outfile;
    system $convert, $outfile, 
      "-resize", "1800x1800", "-quality", "85",
      $preview;
    system $cp, $outfile.".out.pp3", $preview.".out.pp3";
    if ($os eq "Linux") {
#      system "/usr/bin/notify-send", "-i", "rawtherapee", "$file processed";
      system "/scratch/Pictures/Scripts/notify-send.sh", "-i", "rawtherapee", "$file processed";
    } elsif ($os eq "Darwin") {
      system "/usr/local/bin/growlnotify", "-m", "$file processed", "todk.pl";
    }

  } elsif ($infile=~/([^\/]+)%(.+)\.([^\.]+)$/) {
    my $folder=$1;
    my $file=$2;
    my $ext=$3;

    my $preview="/scratch/Pictures/Library/".
      substr($folder, 0, 4)."/".
      substr($folder, 4, 2)."/".
      $folder."/".
      $file."_dt.jpg";
    my $outfile="/scratch/Pictures/Temp/".$file.".".$ext;

    #print LOG "$infile\n$outfile\n$preview\n";

    rename $infile, $outfile;
    system $convert, $outfile, 
      "-resize", "1800x1800", "-quality", "85",
      $preview;
    if ($os eq "Linux") {
#      system "/usr/bin/notify-send", "-i", "rawtherapee", "$file processed";
      system "/scratch/Pictures/Scripts/notify-send.sh", "-i", "rawtherapee", "$file processed";
    } elsif ($os eq "Darwin") {
      system "/usr/local/bin/growlnotify", "-m", "$file processed", "todk.pl";
    }
  }
}

closedir DIR;

#close LOG;

flock(LCK, 0);
close LCK;
